!function(o){const a=o.ti=o.ti||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"","Bookmark must not be empty.":"","Bookmark name":"","Bookmark name already exists.":"","Bookmark name cannot contain space characters.":"","bookmark widget":"","Edit bookmark":"","Enter the bookmark name without spaces.":"",Insert:"","Remove bookmark":"",Update:""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));