/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

/**
 * @file field_ui_add_field.module.css
 */

.field-ui-field-storage-add-form {
  --thumb-size: 4.5rem;
  --color-focus: #26a769;
  --color-gray: #232429;
  --input-fg-color: var(--color-gray);
  --color-blue: #003ecc;
  --color-red: #dc2323;
  --details-box-shadow: 0 2px 0.25rem rgba(0, 0, 0, 0.1);
}

.field-ui-new-storage-wrapper {
  margin-bottom: 0.75rem;
}

.group-field-options-wrapper {
  margin-block: 1.5em;
}

.add-field-container,
.group-field-options {
  display: grid;
  gap: 0.75rem 1.25rem;
  margin-block: 0.625rem;
}

@media (min-width: 45rem) {
  .add-field-container,
  .group-field-options {
    grid-template-columns: repeat(2, 1fr);
  }
}

@media (min-width: 75rem) {
  .add-field-container {
    grid-template-columns: repeat(3, 1fr);
  }
}

@media (min-width: 87.5rem) {
  .add-field-container {
    grid-template-columns: repeat(4, 1fr);
  }
}

.field-option {
  display: grid;
  grid-template-columns: var(--thumb-size) auto;
  align-items: center;
  padding: 0.25rem;
  padding-inline-end: 0.75rem;
  border: 1px solid #dedfe4;
  gap: 0.75rem;
  border-radius: 0.25rem;
}

.field-option__item {
  display: grid;
  grid-template-rows: auto 2fr;
  grid-template-columns: auto 1.1875rem;
  align-items: center;
  width: 100%;
  margin: 0;
  column-gap: 1.25rem;
  padding-block: 0.25rem;
}

.field-option__item > *:not(input) {
  grid-column: 1;
}

.field-option__item > input {
  grid-row: 1 / -1;
  grid-column: 2;
}

.field-option__thumb {
  height: 100%;
  min-height: var(--thumb-size);
  background-color: #f3f4f9;
}

.subfield-option {
  margin-block: 0.625rem;
  padding: 1rem;
  padding-inline-end: 2rem;
  border: 1px solid #d3d4d9;
  border-radius: 0.25rem;
}

.subfield-option .field-option-radio {
  margin-inline-end: 0.4375rem;
}

.subfield-option .item-list ul {
  margin-inline: 0;
}

.field-option,
.subfield-option {
  cursor: pointer;
}

.field-option.focus,
.subfield-option.focus {
  outline: 3px solid var(--color-focus);
  outline-offset: 2px;
}

.field-option.hover,
.subfield-option.hover {
  border-color: var(--color-gray);
  box-shadow:
    inset 0 0 0 1px var(--color-gray),
    var(--details-box-shadow);
}

.field-option:not(.selected, .error):hover .form-boolean,
.subfield-option:not(.selected, .error):hover .form-boolean {
  border-color: var(--input-fg-color);
  box-shadow: inset 0 0 0 1px var(--input-fg-color);
}

.field-option.selected,
.subfield-option.selected {
  border-color: var(--color-blue);
  box-shadow:
    inset 0 0 0 2px var(--color-blue),
    var(--details-box-shadow);
}

.field-option.error,
.subfield-option.error {
  border-color: var(--color-red);
  box-shadow:
    inset 0 0 0 1px var(--color-red),
    var(--details-box-shadow);
}

.field-option .form-item__label.has-error,
.subfield-option .form-item__label.has-error {
  color: currentColor;
}
