!function(n){const i=n.sr=n.sr||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Napravi vezu",Downloadable:"Могуће преузимање","Edit link":"Исправи линк",Link:"Линк","Link image":"Линк слике","Link URL":"УРЛ линк","Link URL must not be empty.":"URL linka ne sme biti prazan.","Move out of a link":"Idi sa veze","Open in a new tab":"Отвори у новој картици","Open link in new tab":"Отвори линк у новом прозору","Scroll to target":"Pomeraj do ciljne veze","This link has no URL":"Линк не садржи УРЛ",Unlink:"Отклони линк"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));