!function(e){const i=e["sr-latn"]=e["sr-latn"]||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable editing":"Nemoguće uređivanje","Enable editing":"Omogući uređivanje","Navigate editable regions":"Pronađi regije za uređivanje","Next editable region":"Sledeća regija za uređivanje","Previous editable region":"Prethodna regija za uređivanje"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));