<?php

declare(strict_types=1);

namespace Drupal\Tests\sqlite\Kernel\sqlite;

use Drupal\KernelTests\Core\Database\DriverSpecificSyntaxTestBase;

/**
 * Tests SQLite syntax interpretation.
 *
 * @group Database
 */
class SyntaxTest extends DriverSpecificSyntaxTestBase {
}
