!function(n){const i=n.nl=n.nl||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Creëer link",Downloadable:"Downloadbaar","Edit link":"Bewerk link",Link:"Link","Link image":"Link afbeelding","Link URL":"Link URL","Link URL must not be empty.":"URL-link mag niet leeg zijn.","Move out of a link":"Uit een link gaan","Open in a new tab":"Open een nieuw tabblad","Open link in new tab":"Open link in nieuw tabblad","Scroll to target":"Scroll naar doel","This link has no URL":"Deze link heeft geen URL",Unlink:"Verwijder link"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));