!function(e){const t=e.ne=e.ne||{};t.dictionary=Object.assign(t.dictionary||{},{"(may require <kbd>Fn</kbd>)":"","%0 of %1":"%0 मध्ये %1",Accept:"",Accessibility:"","Accessibility help":"",Aquamarine:"अव्कवामरिन","Below, you can find a list of keyboard shortcuts that can be used in the editor.":"",Black:"कालो",Blue:"निलो",Cancel:"रद्द गर्नुहोस्","Cannot upload file:":"फाइल अपलोड गर्न मिल्दैन",Clear:"","Click to edit block":"",Close:"","Close contextual balloons, dropdowns, and dialogs":"","Color picker":"","Content editing keystrokes":"","Copy selected content":"","Dim grey":"धमिलो खैरो","Drag to move":"","Dropdown menu":"","Dropdown toolbar":"","Edit block":"ब्लक सम्पादन गर्न","Editor block content toolbar":"","Editor contextual toolbar":"","Editor dialog":"","Editor menu bar":"","Editor toolbar":"","Execute the currently focused button. Executing buttons that interact with the editor content moves the focus back to the content.":"",File:"",Green:"हरियो",Grey:"खैरो","Help Contents. To close this dialog press ESC.":"",HEX:"","Insert a hard break (a new paragraph)":"","Insert a new paragraph directly after a widget":"","Insert a new paragraph directly before a widget":"","Insert a soft break (a <code>&lt;br&gt;</code> element)":"","Insert image with file manager":"","Insert paragraph after block":"","Insert paragraph before block":"","Insert with file manager":"","Keystrokes that can be used when a widget is selected (for example: image, table, etc.)":"","Light blue":"हल्का निलो","Light green":"हल्का हरियो","Light grey":"हल्का खैरो",MENU_BAR_MENU_EDIT:"",MENU_BAR_MENU_FILE:"",MENU_BAR_MENU_FONT:"",MENU_BAR_MENU_FORMAT:"",MENU_BAR_MENU_HELP:"",MENU_BAR_MENU_INSERT:"",MENU_BAR_MENU_TEXT:"",MENU_BAR_MENU_TOOLS:"",MENU_BAR_MENU_VIEW:"","Move focus between form fields (inputs, buttons, etc.)":"","Move focus from an editable area back to the parent widget":"","Move focus in and out of an active dialog window":"","Move focus to the menu bar, navigate between menu bars":"","Move focus to the toolbar, navigate between toolbars":"","Move the caret to allow typing directly after a widget":"","Move the caret to allow typing directly before a widget":"","Navigate through the toolbar or menu bar":"",Next:"अर्को","No results found":"","No searchable items":"","Open the accessibility help dialog":"",Orange:"सुन्तला रंग",Paragraph:"अनुच्छेद","Paste content":"","Paste content as plain text":"",'Please enter a valid color (e.g. "ff0000").':"","Press %0 for help.":"","Press Enter to type after or press Shift + Enter to type before the widget":"",Previous:"अघिल्लो",Purple:"बैंगनी रंग",Red:"रातो",Redo:"रिडु","Remove color":"रंग हटाउन","Replace image with file manager":"","Replace with file manager":"","Restore default":"","Rich Text Editor":"धनी पाठ सम्पादक","Rich Text Editor. Editing area: %0":"",Save:"सुरक्षित गर्नुहोस्","Select all":"","Show more items":"","These keyboard shortcuts allow for quick access to content editing features.":"","Toggle caption off":"","Toggle caption on":"",Turquoise:"त्रकोइस",Undo:"पूर्ववत","Upload in progress":"अपलोड हुदैछ","Use the following keystrokes for more efficient navigation in the CKEditor 5 user interface.":"","User interface and content navigation keystrokes":"",White:"सेतो","Widget toolbar":"","With file manager":"",Yellow:"पहेंलो"}),t.getPluralForm=function(e){return 1!=e}}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));