!function(o){const a=o.bn=o.bn||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"বুকমার্ক","Bookmark must not be empty.":"বুকমার্ক খালি রাখা যাবে না।","Bookmark name":"বুকমার্কের নাম","Bookmark name already exists.":"বুকমার্কের নাম আগে থেকেই বিদ্যমান।","Bookmark name cannot contain space characters.":"বুকমার্কের নামে স্পেস ক্যারেক্টার রাখা যাবে না।","bookmark widget":"বুকমার্ক উইজেট","Edit bookmark":"বুকমার্ক এডিট করুন","Enter the bookmark name without spaces.":"স্পেস ছাড়া বুকমার্কের নাম লিখুন।",Insert:"ঢোকান","Remove bookmark":"বুকমার্ক অপসারণ করুন",Update:"আপডেট করুন"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));