!function(i){const n=i.lt=i.lt||{};n.dictionary=Object.assign(n.dictionary||{},{"Create link":"Sukurti nuorodą",Downloadable:"Parsisiunčiamas","Edit link":"Keisti nuorodą",Link:"Pridėti nuorodą","Link image":"Susieti paveikslėlį","Link URL":"Nuorodos URL","Link URL must not be empty.":"Nuorodos URL negali būti tuščias.","Move out of a link":"Išeiti iš nuorodos","Open in a new tab":"Atverti naujoje kortelėje","Open link in new tab":"Atidaryti nuorodą naujame skirtuke","Scroll to target":"Slinkti iki paskirties vietos","This link has no URL":"Ši nuorda neturi URL",Unlink:"Pašalinti nuorodą"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));