{#
/**
 * @file
 * Default theme implementation for a views UI preview section.
 *
 * Available variables:
 * - title: The human readable section title.
 * - links: A list of contextual links.
 * - content: The content for this section preview.
 *
 * @see template_preprocess_views_ui_view_preview_section()
 *
 * @ingroup themeable
 */
#}
<h1 class="section-title">{{ title }}</h1>
{% if links %}
  <div class="contextual">{{ links }}</div>
{% endif %}
<div class="preview-section">{{ content }}</div>
