!function(i){const e=i.es=i.es||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable editing":"Inhabilitar edición","Enable editing":"Habilitar edición","Navigate editable regions":"Navegar por las zonas editables","Next editable region":"Siguiente zona editable","Previous editable region":"Anterior zona editable"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));