!function(o){const a=o.sk=o.sk||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Záložka","Bookmark must not be empty.":"Záložka nesmie byť prázdna.","Bookmark name":"Názov záložky","Bookmark name already exists.":"Názov záložky už existuje.","Bookmark name cannot contain space characters.":"Názov záložky nemôže obsahovať medzery.","bookmark widget":"widget pre záložky","Edit bookmark":"Upraviť záložku","Enter the bookmark name without spaces.":"Zadajte názov záložky bez medzier.",Insert:"Vložiť","Remove bookmark":"Odstrániť záložku",Update:"Aktualizovať"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));