!function(n){const i=n.fr=n.fr||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Créer un lien",Downloadable:"Fichier téléchargeable","Edit link":"Modifier le lien",Link:"Lien","Link image":"Lien d'image","Link URL":"URL du lien","Link URL must not be empty.":"L'URL du lien ne doit pas être vide.","Move out of a link":"Sortir d'un lien","Open in a new tab":"Ouvrir dans un nouvel onglet","Open link in new tab":"Ouvrir le lien dans un nouvel onglet","Scroll to target":"Faire défiler jusqu'à la cible","This link has no URL":"Ce lien n'a pas d'URL",Unlink:"Supprimer le lien"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));