!function(i){const a=i.fi=i.fi||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Kirjanmerkki","Bookmark must not be empty.":"Kirjanmerkkiä ei saa jättää tyhjäksi.","Bookmark name":"Kirjanmerkin nimi","Bookmark name already exists.":"Kirjanmerkin nimi on jo käytössä.","Bookmark name cannot contain space characters.":"Kirjanmerkin nimessä ei saa olla välilyöntejä.","bookmark widget":"kirjanmerkkien pienoisohjelma","Edit bookmark":"Muokkaa kirjanmerkkiä","Enter the bookmark name without spaces.":"Lisää kirjanmerkin nimi ilman välilyöntejä",Insert:"Liitä","Remove bookmark":"Poista kirjanmerkki",Update:"Päivitä"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));