/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */
/**
 * @file
 * Styles for dropbuttons in the off-canvas dialog.
 *
 * @internal
 */
#drupal-off-canvas-wrapper {
  --off-canvas-dropbutton-height: 1.5rem;
  --off-canvas-dropbutton-primary-background-color: var(--off-canvas-button-background-color);
  --off-canvas-dropbutton-secondary-background-color: var(--off-canvas-button-hover-background-color);
  --off-canvas-dropbutton-border-color: transparent;
  --off-canvas-dropbutton-border-width: 1px;
  --off-canvas-dropbutton-border-radius: 2px;
  --off-canvas-dropbutton-focus-outline-color: var(--off-canvas-focus-outline-color);
  --off-canvas-dropbutton-font-size: 0.75rem;
  --off-canvas-dropbutton-text-color: var(--off-canvas-button-text-color); /* Minimum 4.5:1 contrast ratio against --off-canvas-dropbutton-primary-background-color and --off-canvas-dropbutton-secondary-background-color. */
  --off-canvas-dropbutton-text-color-hover: var(--off-canvas-button-text-color-hover); /* Minimum 4.5:1 contrast ratio against --off-canvas-dropbutton-primary-background-color and --off-canvas-dropbutton-secondary-background-color. */
}
#drupal-off-canvas-wrapper .dropbutton-wrapper {
  margin-block: var(--off-canvas-vertical-spacing-unit);

  /*
     * Styles for when the dropbutton is expanded.
     */
}
#drupal-off-canvas-wrapper .dropbutton-wrapper.open {
  position: relative;
  z-index: 100;
}
#drupal-off-canvas-wrapper .dropbutton-wrapper.open .secondary-action {
  visibility: visible;
}
#drupal-off-canvas-wrapper .dropbutton-wrapper.open .dropbutton-widget {
  border-radius: var(--off-canvas-dropbutton-border-radius) var(--off-canvas-dropbutton-border-radius) 0 0;
}
#drupal-off-canvas-wrapper .dropbutton-wrapper.open .dropbutton-toggle button::before {
  transform: translateY(25%) rotate(225deg);
}
/*
    * Styles for single link variant of dropbutton.
    */
#drupal-off-canvas-wrapper .dropbutton-wrapper.dropbutton-single .dropbutton-widget {
  padding-inline-end: 0;
}
#drupal-off-canvas-wrapper .dropbutton-wrapper.dropbutton-single .dropbutton-action:first-child {
  border-right: solid 1px var(--off-canvas-dropbutton-border-color); /* LTR */
  border-radius: var(--off-canvas-dropbutton-border-radius);
}
#drupal-off-canvas-wrapper .dropbutton-wrapper.dropbutton-single .dropbutton-action:first-child:dir(rtl) {
  border: solid 1px var(--off-canvas-dropbutton-border-color);
}
#drupal-off-canvas-wrapper .dropbutton-wrapper.dropbutton-single .dropbutton-action a {
  justify-content: center;
}
#drupal-off-canvas-wrapper .dropbutton-widget {
  position: relative;
  width: max-content;
  max-width: 100%;
  height: var(--off-canvas-dropbutton-height);
  padding-inline-end: var(--off-canvas-dropbutton-height);
  border-radius: var(--off-canvas-dropbutton-border-radius);
}
#drupal-off-canvas-wrapper .dropbutton {
  height: var(--off-canvas-dropbutton-height);
  margin-block: 0;
  margin-inline-start: 0;
  padding-inline-start: 0;
  list-style: none;
  font-size: var(--off-canvas-dropbutton-font-size);
}
/* This is the button that expands/collapses the secondary options. */
#drupal-off-canvas-wrapper .dropbutton-toggle {
  padding: 0;
  border: 0;
}
#drupal-off-canvas-wrapper .dropbutton-toggle button {
  position: absolute;
  top: 0;
  display: flex;
  align-items: center;
  justify-content: center;
  width: var(--off-canvas-dropbutton-height);
  height: var(--off-canvas-dropbutton-height);
  padding: 0;
  cursor: pointer;
  border-color: var(--off-canvas-dropbutton-border-color);
  border-radius: 0 var(--border-radius) var(--border-radius) 0; /* LTR */
  background: var(--off-canvas-dropbutton-primary-background-color);
  inset-inline-end: 0;
}
#drupal-off-canvas-wrapper .dropbutton-toggle button:focus {
  outline: solid 2px var(--off-canvas-dropbutton-focus-outline-color);
  outline-offset: -2px;
}
#drupal-off-canvas-wrapper .dropbutton-toggle button::before {
  display: block;
  width: 0.375rem;
  height: 0.375rem;
  content: "";
  transform: translateY(-25%) rotate(45deg);
  border-right: solid 2px var(--off-canvas-dropbutton-text-color);
  border-bottom: solid 2px var(--off-canvas-dropbutton-text-color);
}
#drupal-off-canvas-wrapper .dropbutton-toggle button:dir(rtl) {
  border-radius: var(--off-canvas-dropbutton-border-radius) 0 0 var(--off-canvas-dropbutton-border-radius);
}
/* This is the first <li> element in the list of actions. */
#drupal-off-canvas-wrapper .dropbutton-action:first-child {
  margin-inline-end: 2px;
  border: solid var(--off-canvas-dropbutton-border-width) var(--off-canvas-dropbutton-border-color);
  border-radius: var(--off-canvas-dropbutton-border-radius) 0 0 var(--off-canvas-dropbutton-border-radius); /* LTR */
  background: var(--off-canvas-dropbutton-primary-background-color);
}
#drupal-off-canvas-wrapper .dropbutton-action:first-child:dir(rtl) {
  border: solid var(--off-canvas-dropbutton-border-width) var(--off-canvas-dropbutton-border-color);
  border-radius: 0 var(--off-canvas-dropbutton-border-radius) var(--off-canvas-dropbutton-border-radius) 0;
}
#drupal-off-canvas-wrapper .dropbutton-action a {
  display: flex;
  align-items: center;
  min-height: var(--off-canvas-dropbutton-height);
  margin-bottom: -2px;
  padding: 0 0.5625rem;
  -webkit-text-decoration: none;
  text-decoration: none;
  color: var(--off-canvas-dropbutton-text-color);
  font-weight: 600;
}
#drupal-off-canvas-wrapper .dropbutton-action a:hover {
  color: var(--off-canvas-dropbutton-text-color);
}
#drupal-off-canvas-wrapper .dropbutton-action a:focus {
  outline: solid 2px var(--off-canvas-dropbutton-focus-outline-color);
  outline-offset: -1px; /* Overlap parent container by 1px. */
}
/* These are the <li> elements other than the first. */
#drupal-off-canvas-wrapper .secondary-action {
  visibility: hidden;
  width: calc(100% + var(--off-canvas-dropbutton-height));
  margin-top: var(--off-canvas-dropbutton-border-width);
  border-right: var(--off-canvas-dropbutton-border-width) solid var(--off-canvas-dropbutton-border-color);
  border-left: var(--off-canvas-dropbutton-border-width) solid var(--off-canvas-dropbutton-border-color);
  background-color: var(--off-canvas-dropbutton-primary-background-color);
}
#drupal-off-canvas-wrapper .secondary-action:last-child {
  border-bottom: var(--off-canvas-dropbutton-border-width) solid var(--off-canvas-dropbutton-border-color);
}
#drupal-off-canvas-wrapper .secondary-action a:hover {
  color: var(--off-canvas-dropbutton-text-color-hover);
  background-color: var(--off-canvas-dropbutton-secondary-background-color);
}
