!function(a){const e=a.ro=a.ro||{};e.dictionary=Object.assign(e.dictionary||{},{Bookmark:"Marcaj","Bookmark must not be empty.":"Marcajul nu poate fi gol.","Bookmark name":"Numele marcajului","Bookmark name already exists.":"Numele marcajului există deja.","Bookmark name cannot contain space characters.":"Numele marcajului nu poate conține spații.","bookmark widget":"widget marcaj","Edit bookmark":"Editează marcajul","Enter the bookmark name without spaces.":"Introdu numele marcajului fără spații.",Insert:"Inserează","Remove bookmark":"Elimină marcajul",Update:"Actualizează"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));