<?php

declare(strict_types=1);

namespace Drupal\Tests\jsonapi\Functional;

use Drupal\Tests\system\Functional\Module\GenericModuleTestBase;

/**
 * Generic module test for jsonapi.
 *
 * @group jsonapi
 */
class GenericTest extends GenericModuleTestBase {}
