!function(n){const i=n.ro=n.ro||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Crearea unui link",Downloadable:"Descărcabil","Edit link":"Modifică link",Link:"Link","Link image":"Link imagine","Link URL":"Link URL","Link URL must not be empty.":"URL-ul linkului nu trebuie să fie necompletat.","Move out of a link":"Ieșire dintr-un link","Open in a new tab":"Deschide în tab nou","Open link in new tab":"Deschide link în tab nou","Scroll to target":"Derulează până la țintă","This link has no URL":"Acest link nu are niciun URL",Unlink:"Șterge link"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));