!function(n){const i=n.id=n.id||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable editing":"Nonaktifkan penyuntingan","Enable editing":"Aktifkan penyuntingan","Navigate editable regions":"Posisikan bagian-bagian penyuntingan","Next editable region":"Bagian penyuntingan berikutnya","Previous editable region":"Bagian penyuntingan sebelumnya"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));