!function(a){const t=a.lv=a.lv||{};t.dictionary=Object.assign(t.dictionary||{},{Bookmark:"Grāmatzīme","Bookmark must not be empty.":"Grāmatzīmes lauks nedrīkst būt tukšs.","Bookmark name":"Grāmatzīmes nosaukums","Bookmark name already exists.":"Šāds grāmatzīmes nosaukums jau pastāv.","Bookmark name cannot contain space characters.":"Grāmatzīmes nosaukumā nedrīkst būt atstarpes.","bookmark widget":"grāmatzīmju ikona","Edit bookmark":"Rediģēt grāmatzīmi","Enter the bookmark name without spaces.":"Ievadiet grāmatzīmes nosaukumu bez atstarpēm.",Insert:"Ievietot","Remove bookmark":"Aizvākt grāmatzīmi",Update:"Atjaunināt"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));