/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

/**
 * @file
 * Styling of AJAX actions throbber in off-canvas dialog.
 *
 * @internal
 */

#drupal-off-canvas-wrapper .ajax-progress,
#drupal-off-canvas-wrapper .ajax-progress-throbber {
  display: inline-block;
  overflow: hidden;
  width: 0.9375rem;
  height: 0.9375rem;
  margin: 0 0.625rem;
  animation: off-canvas-throbber-spin 1s linear infinite;
  vertical-align: middle;
  border: 2px solid var(--off-canvas-text-color);
  border-top-color: transparent;
  border-radius: 50%;
}

@media (forced-colors: active) {
  #drupal-off-canvas-wrapper .ajax-progress,
  #drupal-off-canvas-wrapper .ajax-progress-throbber {
    border-top-color: transparent;
  }
}

#drupal-off-canvas-wrapper .layout-selection .ajax-progress,
#drupal-off-canvas-wrapper .inline-block-list .ajax-progress,
#drupal-off-canvas-wrapper .layout-selection .ajax-progress-throbber,
#drupal-off-canvas-wrapper .inline-block-list .ajax-progress-throbber {
  position: absolute;
  inset-block-start: 0;
  inset-block-end: 0;
  inset-inline-end: 0;
  margin-block: auto;
}

@keyframes off-canvas-throbber-spin {
  to {
    transform: rotate(360deg);
  }
}
