!function(e){const t=e.hu=e.hu||{};t.dictionary=Object.assign(t.dictionary||{},{"Disable editing":"Szerkesztés letiltása","Enable editing":"Szerkesztés engedélyezése","Navigate editable regions":"Szerkeszthető területek navigálása","Next editable region":"Következő szerkeszthető terület","Previous editable region":"Előző szerkeszthető terület"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));