!function(i){const e=i.ur=i.ur||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable editing":"تدویں غیر فعال کریں","Enable editing":"تدوین فعال کریں","Navigate editable regions":"","Next editable region":"","Previous editable region":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));