/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */

/**
 * @file
 * Styling for the Workspaces overview table.
 */

/** @todo Move to Claro theme before Workspaces is marked stable. */

tr.active-workspace {
  background-color: #ebeae4;
}
