!function(n){const i=n.ti=n.ti||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"መራኽቦ ፍጠር",Downloadable:"ዝረግፍ","Edit link":"መራኽቦ ኣርም",Link:"መራኽቦ","Link image":"","Link URL":"","Link URL must not be empty.":"","Move out of a link":"","Open in a new tab":"","Open link in new tab":"ንመራኽቦ ኣብ ሓዱሽ ታብ ክፈት","Scroll to target":"","This link has no URL":"",Unlink:"መራኽቦ ኣወግድ"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));