!function(i){const e=i.ti=i.ti||{};e.dictionary=Object.assign(e.dictionary||{},{"Disable editing":"","Enable editing":"","Navigate editable regions":"","Next editable region":"","Previous editable region":""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));