---
label: 'Managing content moderation workflows'
top_level: true
related:
  - core.content_structure
---
{% set configuring_workflows_topic = render_var(help_topic_link('content_moderation.configuring_workflows')) %}
{% set changing_states_topic = render_var(help_topic_link('content_moderation.changing_states')) %}
<h2>{% trans %}What is a content moderation workflow?{% endtrans %}</h2>
<p>{% trans %}On some sites, new content and content revisions need to be <em>moderated</em>. That is, they need to pass through several <em>states</em> before becoming visible to site visitors. The collection of states and the definition of the transitions between states is known as a <em>workflow</em>. For example, new content might start out in a <em>Draft</em> state, and then might need to pass through several <em>Review</em> states before it becomes <em>Published</em> on the live site.{% endtrans %}</p>
<p>{% trans %}The core software allows you to configure workflows in which each transition has an associated permission that can be granted to a particular role. See {{ configuring_workflows_topic }} for more information.{% endtrans %}</p>
<p>{% trans %}Users with sufficient permissions can change the workflow state of a particular entity. See {{ changing_states_topic }} for more information.{% endtrans %}</p>
<h2>{% trans %}Overview of content moderation workflows{% endtrans %}</h2>
<ul>
  <li>{% trans %}The core Content Moderation module allows you to expand on core software's "unpublished" and "published" states for content. It allows you to have a published version that is live, but have a separate working copy that is undergoing review before it is published. This is achieved by using workflows to apply different states and transitions to entities as needed.{% endtrans %}</li>
  <li>{% trans %}The core Workflows module allows you to manage workflows with states and transitions.{% endtrans %}</li>
</ul>
<p>{% trans %}See the related topics listed below for specific tasks and background information.{% endtrans %}</p>
<h2>{% trans %}Additional resources{% endtrans %}</h2>
<ul>
  <li><a href="https://www.drupal.org/docs/8/core/modules/content-moderation/overview">{% trans %}On-line documentation about Content Moderation{% endtrans %}</a>
  </li>
</ul>
