!function(e){const i=e.de=e.de||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable editing":"Bearbeitung deaktivieren","Enable editing":"Bearbeitung zulassen","Navigate editable regions":"Durch bearbeitbare Bereiche navigieren","Next editable region":"Nächster bearbeitbarer Bereich","Previous editable region":"Vorheriger bearbeitbarer Bereich"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));