/*
 * DO NOT EDIT THIS FILE.
 * See the following change record for more information,
 * https://www.drupal.org/node/3084859
 * @preserve
 */
/* cspell:ignore wght */
.toolbar-dropdown__item + .toolbar-dropdown__item {
  border-top: 1px solid #d8dfea;
}
.toolbar-dropdown__link {
  display: block;
  padding-inline: var(--admin-toolbar-space-16) var(--admin-toolbar-space-12);
  padding-block: var(--admin-toolbar-space-12);
  -webkit-text-decoration: none;
  text-decoration: none;
  color: var(--admin-toolbar-color-gray-800);
  border-radius: var(--admin-toolbar-space-8);
  font-size: var(--admin-toolbar-font-size-info-sm);
  line-height: var(--admin-toolbar-line-height-info-sm);
  font-variation-settings: "wght" 700;
}
.toolbar-dropdown__link:hover {
  z-index: 20;
  color: var(--admin-toolbar-color-gray-900);
  outline: 2px solid var(--admin-toolbar-color-blue-300);
  outline-offset: var(--toolbar-button-outline-offset);
  background-color: var(--admin-toolbar-color-gray-050);
}
.toolbar-dropdown__link:focus {
  z-index: 10;
  color: var(--admin-toolbar-color-blue-600);
  outline: 2px solid var(--admin-toolbar-color-focus);
  outline-offset: var(--toolbar-button-outline-offset);
  background-color: transparent;
}
.toolbar-dropdown__link:hover:focus {
  background-color: var(--admin-toolbar-color-gray-050);
}
.toolbar-dropdown__list {
  margin: 0;
  padding: 0;
  list-style-type: none;
}
.toolbar-dropdown__menu {
  position: fixed;
  z-index: 1000;
  display: none;
  inline-size: calc(11.25 * var(--admin-toolbar-rem));
  padding: var(--admin-toolbar-space-8) var(--admin-toolbar-space-16);
  border-radius: var(--admin-toolbar-space-12);
  background: white;
  box-shadow: 0 14px 30px 0 rgba(0, 0, 0, 0.1);
}
[data-drupal-dropdown][aria-expanded="true"] + .toolbar-dropdown__menu {
  display: block;
}
