!function(n){const a=n.gl=n.gl||{};a.dictionary=Object.assign(a.dictionary||{},{"Create link":"Crear ligazón",Downloadable:"Descargábel","Edit link":"Editar a ligazón",Link:"Ligar","Link image":"Ligazón da imaxe","Link URL":"URL de ligazón","Link URL must not be empty.":"O URL da ligazón non debe estar baleiro.","Move out of a link":"Saír dunha ligazón","Open in a new tab":"Abrir nunha nova lapela","Open link in new tab":"Abrir a ligazón nunha nova lapela","Scroll to target":"","This link has no URL":"Esta ligazón non ten URL",Unlink:"Desligar"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));