/* cspell:ignore wght */
/**
 * @file
 * Tooltip styles.
 */

@import "../base/media-queries.pcss.css";

.toolbar-tooltip {
  position: fixed;
  z-index: var(--admin-toolbar-z-index-tooltip);
  display: none;
  padding-inline: var(--admin-toolbar-space-12);
  padding-block: var(--admin-toolbar-space-8);
  color: var(--admin-toolbar-color-white);
  border-radius: var(--admin-toolbar-space-8);
  background-color: var(--admin-toolbar-color-gray-990);
  font-size: var(--admin-toolbar-font-size-info-sm);
  font-variation-settings: "wght" 600;
  line-height: var(--admin-toolbar-line-height-info-sm);
}

@media (--admin-toolbar-desktop) {
  [data-drupal-tooltip]:hover + .toolbar-tooltip,
  [data-drupal-tooltip]:focus + .toolbar-tooltip {
    display: block;
  }
}

[data-admin-toolbar="expanded"] [data-drupal-tooltip]:hover + .toolbar-block__title-tooltip {
  display: none;
}
