!function(o){const a=o.el=o.el||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Σελιδοδείκτης","Bookmark must not be empty.":"Ο σελιδοδείκτης δεν πρέπει να είναι κενός.","Bookmark name":"Όνομα σελιδοδείκτη","Bookmark name already exists.":"Το όνομα σελιδοδείκτη υπάρχει ήδη.","Bookmark name cannot contain space characters.":"Το όνομα του σελιδοδείκτη δεν μπορεί να περιέχει χαρακτήρες κενού διαστήματος.","bookmark widget":"widget σελιδοδείκτη","Edit bookmark":"Επεξεργασία σελιδοδείκτη","Enter the bookmark name without spaces.":"Εισαγάγετε το όνομα του σελιδοδείκτη χωρίς κενά.",Insert:"Εισαγωγή","Remove bookmark":"Κατάργηση σελειδοδείκτη",Update:"Ενημέρωση"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));