{#
/**
 * @file
 * Default theme implementation for testing the mapping row style.
 *
 * Available variables:
 * - element: The view content.
 *
 * @see template_preprocess_views_view_mapping_test()
 *
 * @ingroup themeable
 */
#}
{{ element }}
