!function(n){const i=n.kk=n.kk||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"",Downloadable:"","Edit link":"",Link:"","Link image":"","Link URL":"","Link URL must not be empty.":"","Move out of a link":"","Open in a new tab":"","Open link in new tab":"","Scroll to target":"","This link has no URL":"",Unlink:""})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));