!function(o){const a=o["pt-br"]=o["pt-br"]||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Favorito","Bookmark must not be empty.":"O favorito não pode estar vazio.","Bookmark name":"Nome do favorito","Bookmark name already exists.":"O nome do favorito já existe.","Bookmark name cannot contain space characters.":"O nome do favorito não pode conter caracteres de espaço.","bookmark widget":"widget de favorito","Edit bookmark":"Editar favorito","Enter the bookmark name without spaces.":"Insira o nome do favorito sem espaços.",Insert:"Inserir","Remove bookmark":"Remover favorito",Update:"Atualizar"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));