!function(n){const e=n.no=n.no||{};e.dictionary=Object.assign(e.dictionary||{},{"Create link":"Opprett lenke",Downloadable:"Nedlastbar","Edit link":"Rediger lenke",Link:"Lenke","Link image":"Bildelenke","Link URL":"Lenke-URL","Link URL must not be empty.":"Link-URL kan ikke være tom.","Move out of a link":"Flytt ut fra en lenke","Open in a new tab":"Åpne i ny fane","Open link in new tab":"Åpne lenke i ny fane","Scroll to target":"Bla til mål","This link has no URL":"Denne lenken mangler en URL",Unlink:"Fjern lenke"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));