!function(n){const i=n.it=n.it||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Crea un link",Downloadable:"Scaricabile","Edit link":"Modifica collegamento",Link:"Collegamento","Link image":"Collega immagine","Link URL":"URL del collegamento","Link URL must not be empty.":"L'URL del link non può essere lasciato in bianco.","Move out of a link":"Esce da un link","Open in a new tab":"Apri in una nuova scheda","Open link in new tab":"Apri collegamento in nuova scheda","Scroll to target":"Scorri fino alla destinazione","This link has no URL":"Questo collegamento non ha un URL",Unlink:"Elimina collegamento"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));