!function(e){const i=e.hr=e.hr||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable editing":"Onemogući uređivanje","Enable editing":"Omogući uređivanje","Navigate editable regions":"Navigiraj regijama koje je moguće uređivati","Next editable region":"Sljedeća regija koju je moguće uređivati","Previous editable region":"Prethodna regija koju je moguće uređivati"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));