{#
/**
 * @file
 * Theme override to display all the fields in a views row.
 *
 * The reason for this template is to override the theme function provided by
 * views.
 */
#}
{% include '@views/views-view-fields.html.twig' %}
May the force be with you!
