!function(e){const i=e.no=e.no||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable editing":"Deaktivere redigering","Enable editing":"Aktivere redigering","Navigate editable regions":"Naviger redigerbare regioner","Next editable region":"Neste redigerbare region","Previous editable region":"Forrige redigerbare region"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));