!function(a){const o=a.es=a.es||{};o.dictionary=Object.assign(o.dictionary||{},{Bookmark:"Marcador","Bookmark must not be empty.":"El marcador no debe estar vacío.","Bookmark name":"Nombre del marcador","Bookmark name already exists.":"Ya existe el nombre del marcador.","Bookmark name cannot contain space characters.":"El nombre del marcador no puede contener espacios.","bookmark widget":"Módulo interactivo del marcador","Edit bookmark":"Editar marcador","Enter the bookmark name without spaces.":"Introduzca el nombre del marcador sin espacios.",Insert:"Insertar","Remove bookmark":"Eliminar marcador",Update:"Actualizar"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));