!function(i){const a=i.sk=i.sk||{};a.dictionary=Object.assign(a.dictionary||{},{"Disable editing":"Vypnúť režim úprav","Enable editing":"Zapnúť režim úprav","Navigate editable regions":"Navigácia v oblasti úprav","Next editable region":"Nasledujúca oblasť úprav","Previous editable region":"Predchádzajúca oblasť úprav"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));