!function(n){const i=n.pl=n.pl||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"Tworzy link",Downloadable:"Do pobrania","Edit link":"Edytuj odnośnik",Link:"Wstaw odnośnik","Link image":"Wstaw odnośnik do obrazka","Link URL":"Adres URL","Link URL must not be empty.":"Adres URL linku nie może być pusty","Move out of a link":"Umożliwia wyjście z linku","Open in a new tab":"Otwórz w nowej zakładce","Open link in new tab":"Otwórz odnośnik w nowej zakładce","Scroll to target":"Przewiń do celu","This link has no URL":"Nie podano adresu URL odnośnika",Unlink:"Usuń odnośnik"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));