!function(o){const a=o["en-gb"]=o["en-gb"]||{};a.dictionary=Object.assign(a.dictionary||{},{Bookmark:"Bookmark","Bookmark must not be empty.":"Bookmark must not be empty.","Bookmark name":"Bookmark name","Bookmark name already exists.":"Bookmark name already exists.","Bookmark name cannot contain space characters.":"Bookmark name cannot contain space characters.","bookmark widget":"bookmark widget","Edit bookmark":"Edit bookmark","Enter the bookmark name without spaces.":"Enter the bookmark name without spaces.",Insert:"Insert","Remove bookmark":"Remove bookmark",Update:"Update"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));