!function(o){const e=o.cs=o.cs||{};e.dictionary=Object.assign(e.dictionary||{},{Bookmark:"Záložka","Bookmark must not be empty.":"Záložka nesmí být prázdná","Bookmark name":"Název záložky","Bookmark name already exists.":"Záložka s tímto názvem už existuje.","Bookmark name cannot contain space characters.":"Název záložky nemůže obsahovat znaky mezery.","bookmark widget":"widget záložky","Edit bookmark":"Upravit záložku","Enter the bookmark name without spaces.":"Zadejte název záložky bez mezer.",Insert:"Vložit","Remove bookmark":"Odstranit záložku",Update:"Aktualizace"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));