<?php

declare(strict_types=1);

namespace Drupal\Tests\breakpoint\Functional;

use Drupal\Tests\system\Functional\Module\GenericModuleTestBase;

/**
 * Generic module test for breakpoint.
 *
 * @group breakpoint
 */
class GenericTest extends GenericModuleTestBase {}
