!function(a){const i=a.fi=a.fi||{};i.dictionary=Object.assign(i.dictionary||{},{"Disable editing":"Poista muokkaus käytöstä","Enable editing":"Ota muokkaus käyttöön","Navigate editable regions":"Siirry muokattavilla alueilla","Next editable region":"Seuraava muokattava alue","Previous editable region":"Aikaisempi muokattava alue"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));