{#
/**
 * @file
 * Default template for the settings of a table style views display.
 *
 * Available variables:
 * - table: A table of options for each field in this display.
 * - form: Any remaining form fields not included in the table.
 *   - description_markup: An overview for the settings of this display.
 *
 * @see template_preprocess_views_ui_style_plugin_table()
 *
 * @ingroup themeable
 */
#}
{{ form.description_markup }}
{{ table }}
{{ form }}
