!function(a){const i=a.et=a.et||{};i.dictionary=Object.assign(i.dictionary||{},{Bookmark:"Järjehoidja","Bookmark must not be empty.":"Järjehoidja väli ei tohi olla tühi.","Bookmark name":"Järjehoidja nimi","Bookmark name already exists.":"Järjehoidja nimi on juba olemas.","Bookmark name cannot contain space characters.":"Järjehoidja nimi ei tohi sisaldada tühikuid.","bookmark widget":"järjehoidja vidin","Edit bookmark":"Muuda järjehoidjat","Enter the bookmark name without spaces.":"Sisestage järjehoidja nimi ilma tühikuteta.",Insert:"Sisesta","Remove bookmark":"Eemalda järjehoidja",Update:"Uuenda"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));