!function(n){const i=n.az=n.az||{};i.dictionary=Object.assign(i.dictionary||{},{"Create link":"",Downloadable:"Yüklənə bilər","Edit link":"Linki redaktə et",Link:"Əlaqələndir","Link image":"","Link URL":"Linkin URL","Link URL must not be empty.":"","Move out of a link":"","Open in a new tab":"Yeni pəncərədə aç","Open link in new tab":"Linki yeni pəncərədə aç","Scroll to target":"","This link has no URL":"Bu linkdə URL yoxdur",Unlink:"Linki sil"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));