!function(n){const i=n.ug=n.ug||{};i.dictionary=Object.assign(i.dictionary||{},{"Decrease indent":"تارايتىشنى كېمەيت","Increase indent":"تارايتىشنى ئاشۇر"})}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));