/**
 * @file
 * Styles for node preview page.
 */

.node-preview-container {
  position: fixed;
  z-index: 499;
  box-sizing: border-box;
  width: 100%;
  padding-inline-start: var(--drupal-displace-offset-left, 10px);
  padding-inline-end: var(--drupal-displace-offset-right, 10px);
  padding-block: 10px;
}

[dir="rtl"] .node-preview-container {
  padding-inline-start: var(--drupal-displace-offset-right, 10px);
  padding-inline-end: var(--drupal-displace-offset-left, 10px);
}

@media only screen and (min-width: 36em) {
  .node-preview-container .form-type-select {
    margin-left: 25%; /* LTR */
  }
  [dir="rtl"] .node-preview-container .form-type-select {
    margin-right: 25%;
    margin-left: 0;
  }
}
