<?php

/**
 * @file
 * Generic transliteration data for the PhpTransliteration class.
 */

$base = [
  0x00 => 'leoss', 'leong', 'leoj', 'leoch', 'leok', 'leot', 'leop', 'leoh', 'le', 'leg', 'lekk', 'legs', 'len', 'lenj', 'lenh', 'led',
  0x10 => 'lel', 'lelg', 'lelm', 'lelb', 'lels', 'lelt', 'lelp', 'lelh', 'lem', 'leb', 'lebs', 'les', 'less', 'leng', 'lej', 'lech',
  0x20 => 'lek', 'let', 'lep', 'leh', 'lyeo', 'lyeog', 'lyeokk', 'lyeogs', 'lyeon', 'lyeonj', 'lyeonh', 'lyeod', 'lyeol', 'lyeolg', 'lyeolm', 'lyeolb',
  0x30 => 'lyeols', 'lyeolt', 'lyeolp', 'lyeolh', 'lyeom', 'lyeob', 'lyeobs', 'lyeos', 'lyeoss', 'lyeong', 'lyeoj', 'lyeoch', 'lyeok', 'lyeot', 'lyeop', 'lyeoh',
  0x40 => 'lye', 'lyeg', 'lyekk', 'lyegs', 'lyen', 'lyenj', 'lyenh', 'lyed', 'lyel', 'lyelg', 'lyelm', 'lyelb', 'lyels', 'lyelt', 'lyelp', 'lyelh',
  0x50 => 'lyem', 'lyeb', 'lyebs', 'lyes', 'lyess', 'lyeng', 'lyej', 'lyech', 'lyek', 'lyet', 'lyep', 'lyeh', 'lo', 'log', 'lokk', 'logs',
  0x60 => 'lon', 'lonj', 'lonh', 'lod', 'lol', 'lolg', 'lolm', 'lolb', 'lols', 'lolt', 'lolp', 'lolh', 'lom', 'lob', 'lobs', 'los',
  0x70 => 'loss', 'long', 'loj', 'loch', 'lok', 'lot', 'lop', 'loh', 'lwa', 'lwag', 'lwakk', 'lwags', 'lwan', 'lwanj', 'lwanh', 'lwad',
  0x80 => 'lwal', 'lwalg', 'lwalm', 'lwalb', 'lwals', 'lwalt', 'lwalp', 'lwalh', 'lwam', 'lwab', 'lwabs', 'lwas', 'lwass', 'lwang', 'lwaj', 'lwach',
  0x90 => 'lwak', 'lwat', 'lwap', 'lwah', 'lwae', 'lwaeg', 'lwaekk', 'lwaegs', 'lwaen', 'lwaenj', 'lwaenh', 'lwaed', 'lwael', 'lwaelg', 'lwaelm', 'lwaelb',
  0xA0 => 'lwaels', 'lwaelt', 'lwaelp', 'lwaelh', 'lwaem', 'lwaeb', 'lwaebs', 'lwaes', 'lwaess', 'lwaeng', 'lwaej', 'lwaech', 'lwaek', 'lwaet', 'lwaep', 'lwaeh',
  0xB0 => 'loe', 'loeg', 'loekk', 'loegs', 'loen', 'loenj', 'loenh', 'loed', 'loel', 'loelg', 'loelm', 'loelb', 'loels', 'loelt', 'loelp', 'loelh',
  0xC0 => 'loem', 'loeb', 'loebs', 'loes', 'loess', 'loeng', 'loej', 'loech', 'loek', 'loet', 'loep', 'loeh', 'lyo', 'lyog', 'lyokk', 'lyogs',
  0xD0 => 'lyon', 'lyonj', 'lyonh', 'lyod', 'lyol', 'lyolg', 'lyolm', 'lyolb', 'lyols', 'lyolt', 'lyolp', 'lyolh', 'lyom', 'lyob', 'lyobs', 'lyos',
  0xE0 => 'lyoss', 'lyong', 'lyoj', 'lyoch', 'lyok', 'lyot', 'lyop', 'lyoh', 'lu', 'lug', 'lukk', 'lugs', 'lun', 'lunj', 'lunh', 'lud',
  0xF0 => 'lul', 'lulg', 'lulm', 'lulb', 'luls', 'lult', 'lulp', 'lulh', 'lum', 'lub', 'lubs', 'lus', 'luss', 'lung', 'luj', 'luch',
];
