!function(e){const t=e["en-au"]=e["en-au"]||{};t.dictionary=Object.assign(t.dictionary||{},{"(may require <kbd>Fn</kbd>)":"","%0 of %1":"%0 of %1",Accept:"",Accessibility:"","Accessibility help":"",Aquamarine:"Aquamarine","Below, you can find a list of keyboard shortcuts that can be used in the editor.":"",Black:"Black",Blue:"Blue",Cancel:"Cancel","Cannot upload file:":"Cannot upload file:",Clear:"Clear","Click to edit block":"",Close:"","Close contextual balloons, dropdowns, and dialogs":"","Color picker":"Colour picker","Content editing keystrokes":"","Copy selected content":"","Dim grey":"Dim grey","Drag to move":"","Dropdown menu":"","Dropdown toolbar":"Dropdown toolbar","Edit block":"Edit block","Editor block content toolbar":"Editor block content toolbar","Editor contextual toolbar":"Editor contextual toolbar","Editor dialog":"","Editor menu bar":"","Editor toolbar":"Editor toolbar","Execute the currently focused button. Executing buttons that interact with the editor content moves the focus back to the content.":"",File:"",Green:"Green",Grey:"Grey","Help Contents. To close this dialog press ESC.":"",HEX:"","Insert a hard break (a new paragraph)":"","Insert a new paragraph directly after a widget":"","Insert a new paragraph directly before a widget":"","Insert a soft break (a <code>&lt;br&gt;</code> element)":"","Insert image with file manager":"","Insert paragraph after block":"Insert paragraph after block","Insert paragraph before block":"Insert paragraph before block","Insert with file manager":"","Keystrokes that can be used when a widget is selected (for example: image, table, etc.)":"","Light blue":"Light blue","Light green":"Light green","Light grey":"Light grey",MENU_BAR_MENU_EDIT:"Edit",MENU_BAR_MENU_FILE:"",MENU_BAR_MENU_FONT:"",MENU_BAR_MENU_FORMAT:"",MENU_BAR_MENU_HELP:"",MENU_BAR_MENU_INSERT:"Insert",MENU_BAR_MENU_TEXT:"Text",MENU_BAR_MENU_TOOLS:"",MENU_BAR_MENU_VIEW:"","Move focus between form fields (inputs, buttons, etc.)":"","Move focus from an editable area back to the parent widget":"","Move focus in and out of an active dialog window":"","Move focus to the menu bar, navigate between menu bars":"","Move focus to the toolbar, navigate between toolbars":"","Move the caret to allow typing directly after a widget":"","Move the caret to allow typing directly before a widget":"","Navigate through the toolbar or menu bar":"",Next:"Next","No results found":"","No searchable items":"","Open the accessibility help dialog":"",Orange:"Orange",Paragraph:"Paragraph","Paste content":"","Paste content as plain text":"",'Please enter a valid color (e.g. "ff0000").':"","Press %0 for help.":"","Press Enter to type after or press Shift + Enter to type before the widget":"Press Enter to type after or press Shift + Enter to type before the widget",Previous:"Previous",Purple:"Purple",Red:"Red",Redo:"Redo","Remove color":"Remove colour","Replace image with file manager":"","Replace with file manager":"","Restore default":"Restore default","Rich Text Editor":"Rich Text Editor","Rich Text Editor. Editing area: %0":"Rich Text Editor. Editing area: %0",Save:"Save","Select all":"Select all","Show more items":"Show more items","These keyboard shortcuts allow for quick access to content editing features.":"","Toggle caption off":"Toggle caption off","Toggle caption on":"Toggle caption on",Turquoise:"Turquoise",Undo:"Undo","Upload in progress":"Upload in progress","Use the following keystrokes for more efficient navigation in the CKEditor 5 user interface.":"","User interface and content navigation keystrokes":"",White:"White","Widget toolbar":"Widget toolbar","With file manager":"",Yellow:"Yellow"}),t.getPluralForm=function(e){return 1!=e}}(window.CKEDITOR_TRANSLATIONS||(window.CKEDITOR_TRANSLATIONS={}));